@@ -69,6 +69,7 @@ struct _EphyEmbed {
   WebKitURIRequest *delayed_request;
   WebKitWebViewSessionState *delayed_state;
   guint delayed_request_source_id;
+  char *typed_input;
 
   GSList *messages; /* owned EphyEmbedStatusbarMsgs */
   GSList *keys;     /* owned strings */
@@ -87,6 +88,7 @@ struct _EphyEmbed {
   gulong progress_update_handler_id;
   gboolean inspector_loaded;
   gboolean progress_bar_enabled;
+  gboolean first_load_finished;
 };
 
 G_DEFINE_FINAL_TYPE (EphyEmbed, ephy_embed, GTK_TYPE_BOX)
@@ -288,6 +290,20 @@ web_view_title_changed_cb (WebKitWebView *web_view,
   ephy_embed_set_title (embed, webkit_web_view_get_title (web_view));
 }
 
+const char *
+ephy_embed_get_typed_input (EphyEmbed *embed)
+{
+  return embed->typed_input;
+}
+
+void
+ephy_embed_set_typed_input (EphyEmbed  *embed,
+                            const char *input)
+{
+  g_free (embed->typed_input);
+  embed->typed_input = g_strdup (input);
+}
+
 static void
 load_changed_cb (WebKitWebView   *web_view,
                  WebKitLoadEvent  load_event,
@@ -299,11 +315,18 @@ load_changed_cb (WebKitWebView   *web_view,
       break;
     case WEBKIT_LOAD_FINISHED: {
       const char *title = webkit_web_view_get_title (web_view);
+
+      embed->first_load_finished = TRUE;
       if (ephy_web_view_get_is_blank (EPHY_WEB_VIEW (web_view)) || !title || !*title)
         ephy_embed_set_title (embed, NULL);
       break;
     }
     case WEBKIT_LOAD_STARTED:
+      if (embed->first_load_finished) {
+        g_free (embed->typed_input);
+        embed->typed_input = NULL;
+      }
+      break;
     case WEBKIT_LOAD_REDIRECTED:
     default:
       break;
@@ -721,7 +744,7 @@ ephy_embed_constructed (GObject *object)
   embed->overlay = gtk_overlay_new ();
 
   gtk_widget_set_vexpand (embed->overlay, TRUE);
-  gtk_overlay_set_child (GTK_OVERLAY (embed->overlay), GTK_WIDGET (embed->web_view));
+  gtk_overlay_set_child (GTK_OVERLAY (embed->overlay), gtk_graphics_offload_new (GTK_WIDGET (embed->web_view)));
 
   /* Floating message popup for fullscreen mode. */
   embed->fullscreen_message_label = gtk_label_new (NULL);
@@ -812,6 +835,7 @@ ephy_embed_init (EphyEmbed *embed)
   embed->seq_message_id = 1;
   embed->tab_message_id = ephy_embed_statusbar_get_context_id (embed, EPHY_EMBED_STATUSBAR_TAB_MESSAGE_CONTEXT_DESCRIPTION);
   embed->inspector_loaded = FALSE;
+  embed->first_load_finished = FALSE;
 }
 
 /**
