@@ -41,6 +41,7 @@ struct _EphyFindToolbar {
   WebKitFindController *controller;
   GtkWidget *search_bar;
   GtkWidget *entry;
+  GtkWidget *options_button;
   GtkWidget *next;
   GtkWidget *prev;
   guint num_matches;
@@ -48,9 +49,11 @@ struct _EphyFindToolbar {
   guint find_again_source_id;
   guint find_source_id;
   char *find_string;
+  gboolean case_sensitive;
+  gboolean word_action;
 };
 
-G_DEFINE_TYPE (EphyFindToolbar, ephy_find_toolbar, ADW_TYPE_BIN)
+G_DEFINE_FINAL_TYPE (EphyFindToolbar, ephy_find_toolbar, ADW_TYPE_BIN)
 
 enum {
   PROP_0,
@@ -121,7 +124,7 @@ static void
 real_find (EphyFindToolbar   *toolbar,
            EphyFindDirection  direction)
 {
-  WebKitFindOptions options = WEBKIT_FIND_OPTIONS_CASE_INSENSITIVE;
+  WebKitFindOptions options = WEBKIT_FIND_OPTIONS_NONE;
 
   if (!g_strcmp0 (toolbar->find_string, ""))
     return;
@@ -129,6 +132,12 @@ real_find (EphyFindToolbar   *toolbar,
   if (direction == EPHY_FIND_DIRECTION_PREV)
     options |= WEBKIT_FIND_OPTIONS_BACKWARDS;
 
+  if (!toolbar->case_sensitive)
+    options |= WEBKIT_FIND_OPTIONS_CASE_INSENSITIVE;
+
+  if (toolbar->word_action)
+    options |= WEBKIT_FIND_OPTIONS_AT_WORD_STARTS;
+
   webkit_find_controller_count_matches (toolbar->controller, toolbar->find_string, options, G_MAXUINT);
   webkit_find_controller_search (toolbar->controller, toolbar->find_string, options, G_MAXUINT);
 }
@@ -201,15 +210,54 @@ search_entry_changed_cb (GtkEditable     *entry,
   update_find_string (toolbar);
 }
 
+static gboolean
+key_pressed_cb (EphyFindToolbar       *toolbar,
+                guint                  keyval,
+                guint                  keycode,
+                GdkModifierType        state,
+                GtkEventControllerKey *key_controller)
+{
+  if (keyval == GDK_KEY_Escape) {
+    ephy_find_toolbar_close (toolbar);
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
 static void
 ephy_find_toolbar_load_changed_cb (WebKitWebView   *web_view,
                                    WebKitLoadEvent  load_event,
                                    EphyFindToolbar *toolbar)
 {
-  if (load_event == WEBKIT_LOAD_STARTED &&
-      gtk_search_bar_get_search_mode (GTK_SEARCH_BAR (toolbar->search_bar))) {
+  const char *address = ephy_web_view_get_display_address (EPHY_WEB_VIEW (web_view));
+  gboolean is_blank = ephy_web_view_get_is_blank (EPHY_WEB_VIEW (web_view));
+  gboolean is_internal_page = g_str_has_prefix (address, "about:") || g_str_has_prefix (address, "ephy-about:");
+
+  if (load_event == WEBKIT_LOAD_STARTED && (is_blank || is_internal_page))
     ephy_find_toolbar_close (toolbar);
-  }
+}
+
+static void
+on_case_senstive (GSimpleAction   *action,
+                  GVariant        *value,
+                  EphyFindToolbar *toolbar)
+{
+  toolbar->case_sensitive = g_variant_get_boolean (value);
+  g_simple_action_set_state (action, value);
+
+  update_find_string (toolbar);
+}
+
+static void
+on_word_action (GSimpleAction   *action,
+                GVariant        *value,
+                EphyFindToolbar *toolbar)
+{
+  toolbar->word_action = g_variant_get_boolean (value);
+  g_simple_action_set_state (action, value);
+
+  update_find_string (toolbar);
 }
 
 static void
@@ -217,6 +265,12 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 {
   GtkWidget *clamp;
   GtkWidget *box;
+  GtkWidget *settings_popover;
+  GtkEventController *event_controller;
+  GMenu *model;
+  g_autoptr (GSimpleActionGroup) group = NULL;
+  g_autoptr (GSimpleAction) case_action = NULL;
+  g_autoptr (GSimpleAction) word_action = NULL;
 
   toolbar->search_bar = gtk_search_bar_new ();
   adw_bin_set_child (ADW_BIN (toolbar), toolbar->search_bar);
@@ -249,6 +303,29 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
   gtk_box_append (GTK_BOX (box), toolbar->next);
   gtk_widget_set_sensitive (toolbar->next, FALSE);
 
+  /* Options */
+  toolbar->options_button = gtk_menu_button_new ();
+  gtk_menu_button_set_icon_name (GTK_MENU_BUTTON (toolbar->options_button), "emblem-system-symbolic");
+
+  group = g_simple_action_group_new ();
+  case_action = g_simple_action_new_stateful ("case-sensitive", NULL, g_variant_new_boolean (FALSE));
+  g_action_map_add_action (G_ACTION_MAP (group), G_ACTION (case_action));
+  g_signal_connect (case_action, "change-state", G_CALLBACK (on_case_senstive), toolbar);
+
+  word_action = g_simple_action_new_stateful ("match-whole-word", NULL, g_variant_new_boolean (FALSE));
+  g_action_map_add_action (G_ACTION_MAP (group), G_ACTION (word_action));
+  g_signal_connect (word_action, "change-state", G_CALLBACK (on_word_action), toolbar);
+
+  gtk_widget_insert_action_group (GTK_WIDGET (toolbar), "search-options", G_ACTION_GROUP (group));
+
+  model = g_menu_new ();
+  g_menu_append_item (model, g_menu_item_new (_("_Case Sensitive"), "search-options.case-sensitive"));
+  g_menu_append_item (model, g_menu_item_new (_("Match Whole _Word Only"), "search-options.match-whole-word"));
+  settings_popover = gtk_popover_menu_new_from_model (G_MENU_MODEL (model));
+  gtk_menu_button_set_popover (GTK_MENU_BUTTON (toolbar->options_button), settings_popover);
+  gtk_widget_set_tooltip_text (toolbar->options_button, _("Search Options"));
+  gtk_box_append (GTK_BOX (box), toolbar->options_button);
+
   /* connect signals */
   g_signal_connect_after (toolbar->entry, "changed",
                           G_CALLBACK (search_entry_changed_cb), toolbar);
@@ -266,6 +343,12 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
   gtk_search_bar_connect_entry (GTK_SEARCH_BAR (toolbar->search_bar),
                                 GTK_EDITABLE (toolbar->entry));
 
+  /* Event controller for Escape key */
+  event_controller = gtk_event_controller_key_new ();
+  gtk_widget_add_controller (GTK_WIDGET (toolbar), event_controller);
+  g_signal_connect_swapped (event_controller, "key-pressed",
+                            G_CALLBACK (key_pressed_cb), toolbar);
+
   search_entry_changed_cb (GTK_EDITABLE (toolbar->entry), toolbar);
 
   toolbar->cancellable = g_cancellable_new ();
@@ -404,9 +487,9 @@ ephy_find_toolbar_set_web_view (EphyFindToolbar *toolbar,
     g_signal_connect_object (toolbar->controller, "counted_matches",
                              G_CALLBACK (counted_matches_cb),
                              toolbar, 0);
-    g_signal_connect (web_view, "load-changed",
-                      G_CALLBACK (ephy_find_toolbar_load_changed_cb),
-                      toolbar);
+    g_signal_connect_object (web_view, "load-changed",
+                             G_CALLBACK (ephy_find_toolbar_load_changed_cb),
+                             toolbar, 0);
 
     clear_status (toolbar);
   }
