@@ -58,6 +58,9 @@ typedef enum
 #define EPHY_PREFS_UI_EXPAND_TABS_BAR            "expand-tabs-bar"
 #define EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY "tabs-bar-visibility-policy"
 #define EPHY_PREFS_UI_KEEP_WINDOW_OPEN           "keep-window-open"
+#define EPHY_PREFS_UI_BOTTOM_URL_BAR             "bottom-url-bar"
+#define EPHY_PREFS_UI_WEBKIT_FEATURES_PANEL      "webkit-features-page"
+#define EPHY_PREFS_UI_WEBKIT_INTERNAL_FEATURES   "webkit-features-page-show-internal"
 
 #define EPHY_PREFS_READER_SCHEMA                 "org.gnome.Epiphany.reader"
 #define EPHY_PREFS_READER_FONT_STYLE             "font-style"
@@ -107,8 +110,11 @@ static const char * const ephy_prefs_state_schema[] = {
 #define EPHY_PREFS_WEB_ENABLE_WEBEXTENSIONS         "enable-webextensions"
 #define EPHY_PREFS_WEB_WEBEXTENSIONS_ACTIVE         "webextensions-active"
 #define EPHY_PREFS_WEB_SHOW_DEVELOPER_ACTIONS       "show-developer-actions"
+#define EPHY_PREFS_WEB_ALWAYS_SHOW_FULL_URL         "always-show-full-url"
 #define EPHY_PREFS_WEB_ENABLE_NAVIGATION_GESTURES   "enable-navigation-gestures"
 #define EPHY_PREFS_WEB_AUTOFILL_DATA                "autofill-data"
+#define EPHY_PREFS_WEB_AUTO_OPEN_SCHEMES            "auto-open-schemes"
+#define EPHY_PREFS_WEB_READER_MODE_ZOOM_LEVEL       "reader-mode-zoom-level"
 
 static const char * const ephy_prefs_web_schema[] = {
   EPHY_PREFS_WEB_FONT_MIN_SIZE,
@@ -138,7 +144,9 @@ static const char * const ephy_prefs_web_schema[] = {
   EPHY_PREFS_WEB_SWITCH_TO_NEW_TAB,
   EPHY_PREFS_WEB_ENABLE_WEBEXTENSIONS,
   EPHY_PREFS_WEB_SHOW_DEVELOPER_ACTIONS,
+  EPHY_PREFS_WEB_ALWAYS_SHOW_FULL_URL,
   EPHY_PREFS_WEB_AUTOFILL_DATA,
+  EPHY_PREFS_WEB_READER_MODE_ZOOM_LEVEL,
 };
 
 #define EPHY_PREFS_SCHEMA                             "org.gnome.Epiphany"
@@ -154,7 +162,8 @@ static const char * const ephy_prefs_web_schema[] = {
 #define EPHY_PREFS_ASK_FOR_DEFAULT                    "ask-for-default"
 #define EPHY_PREFS_START_IN_INCOGNITO_MODE            "start-in-incognito-mode"
 #define EPHY_PREFS_ACTIVE_CLEAR_DATA_ITEMS            "active-clear-data-items"
-#define EPHY_PREFS_USE_GOOGLE_SEARCH_SUGGESTIONS      "use-google-search-suggestions"
+#define EPHY_PREFS_INCOGNITO_SEARCH_ENGINE            "incognito-search-engine"
+#define EPHY_PREFS_USE_SEARCH_SUGGESTIONS             "use-search-suggestions"
 
 #define EPHY_PREFS_LOCKDOWN_SCHEMA            "org.gnome.Epiphany.lockdown"
 #define EPHY_PREFS_LOCKDOWN_FULLSCREEN        "disable-fullscreen"
