@@ -329,3 +329,29 @@ ephy_strv_remove (const char * const *strv,
 
   return new_strv;
 }
+
+char **
+ephy_strv_append (const char * const *strv,
+                  const char         *str)
+{
+  char **new_strv;
+  char **n;
+  const char * const *s;
+  guint len;
+
+  /* Needs room for one more string than before, plus one for trailing NULL. */
+  len = g_strv_length ((char **)strv);
+  new_strv = g_malloc ((len + 1 + 1) * sizeof (char *));
+  n = new_strv;
+  s = strv;
+
+  while (*s != NULL) {
+    *n = g_strdup (*s);
+    n++;
+    s++;
+  }
+  new_strv[len] = g_strdup (str);
+  new_strv[len + 1] = NULL;
+
+  return new_strv;
+}
