@@ -28,7 +28,8 @@
 struct _EphyOpenTabsManager {
   GObject parent_instance;
 
-  EphyTabsCatalog *catalog;
+  EphyTabsCatalog *catalog; /* unowned,
+                             * because the EphyTabsCatalog is the EphyShell, which owns us. */
 
   /* A list of EphyOpenTabsRecord objects describing the open tabs
    * of other sync clients. This is updated at every sync. */
@@ -59,9 +60,7 @@ ephy_open_tabs_manager_set_property (GObject      *object,
 
   switch (prop_id) {
     case PROP_TABS_CATALOG:
-      if (self->catalog)
-        g_object_unref (self->catalog);
-      self->catalog = g_object_ref (g_value_get_object (value));
+      self->catalog = g_value_get_object (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -95,16 +94,6 @@ ephy_open_tabs_manager_finalize (GObject *object)
   G_OBJECT_CLASS (ephy_open_tabs_manager_parent_class)->finalize (object);
 }
 
-static void
-ephy_open_tabs_manager_dispose (GObject *object)
-{
-  EphyOpenTabsManager *self = EPHY_OPEN_TABS_MANAGER (object);
-
-  g_clear_object (&self->catalog);
-
-  G_OBJECT_CLASS (ephy_open_tabs_manager_parent_class)->dispose (object);
-}
-
 static void
 ephy_open_tabs_manager_class_init (EphyOpenTabsManagerClass *klass)
 {
@@ -112,7 +101,6 @@ ephy_open_tabs_manager_class_init (EphyOpenTabsManagerClass *klass)
 
   object_class->set_property = ephy_open_tabs_manager_set_property;
   object_class->get_property = ephy_open_tabs_manager_get_property;
-  object_class->dispose = ephy_open_tabs_manager_dispose;
   object_class->finalize = ephy_open_tabs_manager_finalize;
 
   obj_properties[PROP_TABS_CATALOG] =
