@@ -70,6 +70,14 @@ struct _EphyPasswordManager {
   GHashTable *cache;
 };
 
+enum {
+  SYNCHRONIZABLE_DELETED,
+  SYNCHRONIZABLE_MODIFIED,
+  LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL];
+
 static void ephy_password_manager_forget_record (EphyPasswordManager *self,
                                                  EphyPasswordRecord  *record,
                                                  EphyPasswordRecord  *replacement,
@@ -90,8 +98,9 @@ typedef struct {
 
 typedef struct {
   EphyPasswordManager *manager;
+  char *username;
   char *password;
-} UpdatePasswordAsyncData;
+} UpdateCredentialsAsyncData;
 
 typedef struct {
   EphyPasswordManager *manager;
@@ -130,26 +139,29 @@ query_async_data_free (QueryAsyncData *data)
   g_free (data);
 }
 
-static UpdatePasswordAsyncData *
-update_password_async_data_new (EphyPasswordManager *manager,
-                                const char          *password)
+static UpdateCredentialsAsyncData *
+update_credentials_async_data_new (EphyPasswordManager *manager,
+                                   const char          *username,
+                                   const char          *password)
 {
-  UpdatePasswordAsyncData *data;
+  UpdateCredentialsAsyncData *data;
 
-  data = g_new0 (UpdatePasswordAsyncData, 1);
+  data = g_new0 (UpdateCredentialsAsyncData, 1);
   data->manager = g_object_ref (manager);
+  data->username = g_strdup (username);
   data->password = g_strdup (password);
 
   return data;
 }
 
 static void
-update_password_async_data_free (UpdatePasswordAsyncData *data)
+update_credentials_async_data_free (UpdateCredentialsAsyncData *data)
 {
   g_assert (data);
 
   g_object_unref (data->manager);
-  g_free (data->password);
+  g_clear_pointer (&data->username, g_free);
+  g_clear_pointer (&data->password, g_free);
   g_free (data);
 }
 
@@ -357,6 +369,12 @@ ephy_password_manager_class_init (EphyPasswordManagerClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
   object_class->dispose = ephy_password_manager_dispose;
+
+  signals[SYNCHRONIZABLE_DELETED] = g_signal_lookup ("synchronizable-deleted",
+                                                     EPHY_TYPE_SYNCHRONIZABLE_MANAGER);
+
+  signals[SYNCHRONIZABLE_MODIFIED] = g_signal_lookup ("synchronizable-modified",
+                                                      EPHY_TYPE_SYNCHRONIZABLE_MANAGER);
 }
 
 static void
@@ -493,10 +511,10 @@ deduplicate_records (EphyPasswordManager *manager,
 }
 
 static void
-update_password_cb (GList    *records,
-                    gpointer  user_data)
+update_credentials_cb (GList    *records,
+                       gpointer  user_data)
 {
-  UpdatePasswordAsyncData *data = (UpdatePasswordAsyncData *)user_data;
+  UpdateCredentialsAsyncData *data = (UpdateCredentialsAsyncData *)user_data;
   EphyPasswordRecord *record;
 
   /* Since we didn't include ID in our query, there could be multiple records
@@ -507,14 +525,15 @@ update_password_cb (GList    *records,
 
   if (records) {
     record = EPHY_PASSWORD_RECORD (records->data);
+    ephy_password_record_set_username (record, data->username);
     ephy_password_record_set_password (record, data->password);
     ephy_password_manager_store_record (data->manager, record);
-    g_signal_emit_by_name (data->manager, "synchronizable-modified", record, FALSE);
+    g_signal_emit (data->manager, signals[SYNCHRONIZABLE_MODIFIED], 0, record, FALSE);
   } else {
     LOG ("Attempted to update password record that doesn't exist (likely Epiphany bug)");
   }
 
-  update_password_async_data_free (data);
+  update_credentials_async_data_free (data);
 }
 
 void
@@ -522,6 +541,7 @@ ephy_password_manager_save (EphyPasswordManager *self,
                             const char          *origin,
                             const char          *target_origin,
                             const char          *username,
+                            const char          *new_username,
                             const char          *password,
                             const char          *username_field,
                             const char          *password_field,
@@ -535,15 +555,7 @@ ephy_password_manager_save (EphyPasswordManager *self,
   g_assert (EPHY_IS_PASSWORD_MANAGER (self));
   g_assert (origin);
   g_assert (target_origin);
-
-  /* Although we don't enforce these requirements for Firefox passwords,
-   * any password saved by Epiphany must have both password and
-   * password_field, so that it can be filled in. username and
-   * username_field are optional, but if one is provided, both must be.
-   */
   g_assert (password);
-  g_assert (password_field);
-  g_assert ((username && username_field) || (!username && !username_field));
 
   if (!is_new) {
     LOG ("Updating password for (%s, %s, %s, %s, %s)",
@@ -551,8 +563,8 @@ ephy_password_manager_save (EphyPasswordManager *self,
     ephy_password_manager_query (self, NULL,
                                  origin, target_origin, username,
                                  username_field, password_field,
-                                 update_password_cb,
-                                 update_password_async_data_new (self, password));
+                                 update_credentials_cb,
+                                 update_credentials_async_data_new (self, new_username, password));
     return;
   }
 
@@ -564,7 +576,7 @@ ephy_password_manager_save (EphyPasswordManager *self,
                                      username_field, password_field,
                                      timestamp, timestamp);
   ephy_password_manager_store_record (self, record);
-  g_signal_emit_by_name (self, "synchronizable-modified", record, FALSE);
+  g_signal_emit (self, signals[SYNCHRONIZABLE_MODIFIED], 0, record, FALSE);
 
   g_free (uuid);
   g_free (id);
@@ -822,7 +834,7 @@ forget_cb (GList    *records,
   g_assert (g_list_length (records) == 1);
 
   record = EPHY_PASSWORD_RECORD (records->data);
-  g_signal_emit_by_name (self, "synchronizable-deleted", record);
+  g_signal_emit (self, signals[SYNCHRONIZABLE_DELETED], 0, record);
   ephy_password_manager_forget_record (self, record, NULL, task);
 }
 
@@ -870,7 +882,7 @@ forget_all_cb (GList    *records,
                           (GAsyncReadyCallback)secret_password_clear_cb, NULL);
 
   for (GList *l = records; l && l->data; l = l->next)
-    g_signal_emit_by_name (self, "synchronizable-deleted", l->data);
+    g_signal_emit (self, signals[SYNCHRONIZABLE_DELETED], 0, l->data);
 
   ephy_password_manager_cache_clear (self);
 
@@ -1109,7 +1121,7 @@ ephy_password_manager_handle_initial_merge (EphyPasswordManager *self,
         local_timestamp = ephy_password_record_get_time_password_changed (record);
         if (local_timestamp > remote_timestamp) {
           /* Local record is newer. Keep it, upload it and delete remote record from server. */
-          g_signal_emit_by_name (self, "synchronizable-deleted", l->data);
+          g_signal_emit (self, signals[SYNCHRONIZABLE_DELETED], 0, l->data);
         } else {
           /* Remote record is newer. Forget local record and store remote record. */
           ephy_password_manager_forget_record (self, record, l->data, NULL);
@@ -1205,7 +1217,7 @@ ephy_password_manager_handle_regular_merge (EphyPasswordManager  *self,
         if (local_timestamp > remote_timestamp) {
           /* Local record is newer. Keep it, upload it and delete remote record from server. */
           g_ptr_array_add (to_upload, g_object_ref (record));
-          g_signal_emit_by_name (self, "synchronizable-deleted", l->data);
+          g_signal_emit (self, signals[SYNCHRONIZABLE_DELETED], 0, l->data);
         } else {
           /* Remote record is newer. Forget local record and store remote record. */
           ephy_password_manager_forget_record (self, record, l->data, NULL);
