@@ -37,7 +37,6 @@
 struct _EphyActionBarEnd {
   GtkBox parent_instance;
 
-  GtkWidget *bookmark_button;
   GtkWidget *bookmarks_button;
   GtkWidget *downloads_revealer;
   GtkWidget *downloads_button;
@@ -224,17 +223,6 @@ on_bookmarks_button (GtkButton *button,
   ephy_window_toggle_bookmarks (EPHY_WINDOW (parent_window));
 }
 
-static void
-on_bookmark_button_clicked (GtkButton *button,
-                            gpointer   user_data)
-{
-  GtkWidget *window = GTK_WIDGET (gtk_widget_get_root (GTK_WIDGET (button)));
-  GtkWidget *dialog;
-
-  dialog = ephy_bookmark_properties_new_for_window (EPHY_WINDOW (window));
-  adw_dialog_present (ADW_DIALOG (dialog), window);
-}
-
 static void
 ephy_action_bar_end_class_init (EphyActionBarEndClass *klass)
 {
@@ -243,9 +231,6 @@ ephy_action_bar_end_class_init (EphyActionBarEndClass *klass)
   gtk_widget_class_set_template_from_resource (widget_class,
                                                "/org/gnome/epiphany/gtk/action-bar-end.ui");
 
-  gtk_widget_class_bind_template_child (widget_class,
-                                        EphyActionBarEnd,
-                                        bookmark_button);
   gtk_widget_class_bind_template_child (widget_class,
                                         EphyActionBarEnd,
                                         bookmarks_button);
@@ -343,8 +328,6 @@ ephy_action_bar_end_init (EphyActionBarEnd *action_bar_end)
                            G_CALLBACK (show_downloads_cb),
                            object, 0);
 
-  g_signal_connect_object (action_bar_end->bookmark_button, "clicked", G_CALLBACK (on_bookmark_button_clicked), action_bar_end, 0);
-
   extension_manager = ephy_web_extension_manager_get_default ();
   g_signal_connect_object (extension_manager, "show-browser-action",
                            G_CALLBACK (show_browser_action_cb),
@@ -384,36 +367,6 @@ ephy_action_bar_end_get_downloads_revealer (EphyActionBarEnd *action_bar_end)
   return action_bar_end->downloads_revealer;
 }
 
-void
-ephy_action_bar_end_set_bookmark_icon_state (EphyActionBarEnd      *action_bar_end,
-                                             EphyBookmarkIconState  state)
-{
-  g_assert (EPHY_IS_ACTION_BAR_END (action_bar_end));
-
-  switch (state) {
-    case EPHY_BOOKMARK_ICON_HIDDEN:
-      gtk_widget_set_visible (action_bar_end->bookmark_button, FALSE);
-      break;
-    case EPHY_BOOKMARK_ICON_EMPTY:
-      gtk_widget_set_visible (action_bar_end->bookmark_button, TRUE);
-      gtk_button_set_icon_name (GTK_BUTTON (action_bar_end->bookmark_button),
-                                "ephy-non-starred-symbolic");
-      /* Translators: tooltip for the empty bookmark button */
-      gtk_widget_set_tooltip_text (action_bar_end->bookmark_button, _("Bookmark Page"));
-      break;
-    case EPHY_BOOKMARK_ICON_BOOKMARKED:
-      gtk_widget_set_visible (action_bar_end->bookmark_button, TRUE);
-      gtk_button_set_icon_name (GTK_BUTTON (action_bar_end->bookmark_button),
-                                "ephy-starred-symbolic");
-
-      /* Translators: tooltip for the bookmarked button */
-      gtk_widget_set_tooltip_text (action_bar_end->bookmark_button, _("Edit Bookmark"));
-      break;
-    default:
-      g_assert_not_reached ();
-  }
-}
-
 GtkWidget *
 create_browser_action_item_widget (EphyBrowserAction *action,
                                    gpointer           user_data)
@@ -451,13 +404,3 @@ set_browser_actions (EphyActionBarEnd *action_bar_end,
 
   browser_actions_items_changed_cb (G_LIST_MODEL (browser_actions), 0, 0, 0, action_bar_end);
 }
-
-void
-ephy_action_bar_end_set_adaptive_mode (EphyActionBarEnd *action_bar_end,
-                                       EphyAdaptiveMode  adaptive_mode)
-{
-  gtk_widget_set_visible (action_bar_end->bookmark_button,
-                          adaptive_mode == EPHY_ADAPTIVE_MODE_NORMAL);
-  gtk_widget_set_visible (action_bar_end->overview_button,
-                          adaptive_mode == EPHY_ADAPTIVE_MODE_NORMAL);
-}
