@@ -34,10 +34,10 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <webkit2/webkit2.h>
+#include <webkit/webkit.h>
 
 struct _EphyEncodingDialog {
-  GtkDialog parent_instance;
+  AdwDialog parent_instance;
 
   EphyEncodings *encodings;
   EphyWindow *window;
@@ -70,7 +70,7 @@ enum {
 
 static GParamSpec *obj_properties[LAST_PROP];
 
-G_DEFINE_TYPE (EphyEncodingDialog, ephy_encoding_dialog, GTK_TYPE_DIALOG)
+G_DEFINE_FINAL_TYPE (EphyEncodingDialog, ephy_encoding_dialog, ADW_TYPE_DIALOG)
 
 static void
 select_encoding_row (GtkListBox   *list_box,
@@ -87,7 +87,7 @@ select_encoding_row (GtkListBox   *list_box,
     EphyEncoding *ephy_encoding;
     const char *encoding_string = NULL;
 
-    ephy_encoding_row = EPHY_ENCODING_ROW (gtk_bin_get_child (GTK_BIN (row)));
+    ephy_encoding_row = EPHY_ENCODING_ROW (gtk_list_box_row_get_child (row));
     ephy_encoding = ephy_encoding_row_get_encoding (ephy_encoding_row);
     encoding_string = ephy_encoding_get_encoding (ephy_encoding);
 
@@ -102,6 +102,27 @@ select_encoding_row (GtkListBox   *list_box,
   }
 }
 
+static void
+clean_selected_list_box (GtkListBox *list_box)
+{
+  GtkListBoxRow *row;
+  int i = 0;
+
+  while ((row = gtk_list_box_get_row_at_index (list_box, i++))) {
+    EphyEncodingRow *ephy_encoding_row =
+      EPHY_ENCODING_ROW (gtk_list_box_row_get_child (GTK_LIST_BOX_ROW (row)));
+    ephy_encoding_row_set_selected (ephy_encoding_row, FALSE);
+  }
+}
+
+static void
+clean_selected (EphyEncodingDialog *dialog)
+{
+  clean_selected_list_box (dialog->list_box);
+  clean_selected_list_box (dialog->recent_list_box);
+  clean_selected_list_box (dialog->related_list_box);
+}
+
 static void
 sync_encoding_against_embed (EphyEncodingDialog *dialog)
 {
@@ -117,6 +138,8 @@ sync_encoding_against_embed (EphyEncodingDialog *dialog)
   encoding = webkit_web_view_get_custom_charset (view);
   is_automatic = encoding == NULL;
 
+  clean_selected (dialog);
+
   if (!is_automatic) {
     EphyEncoding *node;
 
@@ -211,35 +234,6 @@ activate_choice (EphyEncodingDialog *dialog)
   }
 }
 
-static void
-ephy_encoding_dialog_response_cb (GtkWidget          *widget,
-                                  int                 response,
-                                  EphyEncodingDialog *dialog)
-{
-  gtk_widget_destroy (GTK_WIDGET (dialog));
-}
-
-static void
-clean_selected_list_box (GtkListBox *list_box)
-{
-  GtkListBoxRow *row;
-  int i = 0;
-
-  while ((row = gtk_list_box_get_row_at_index (list_box, i++))) {
-    EphyEncodingRow *ephy_encoding_row =
-      EPHY_ENCODING_ROW (gtk_bin_get_child (GTK_BIN (row)));
-    ephy_encoding_row_set_selected (ephy_encoding_row, FALSE);
-  }
-}
-
-static void
-clean_selected (EphyEncodingDialog *dialog)
-{
-  clean_selected_list_box (dialog->list_box);
-  clean_selected_list_box (dialog->recent_list_box);
-  clean_selected_list_box (dialog->related_list_box);
-}
-
 static void
 row_activated_cb (GtkListBox         *box,
                   GtkListBoxRow      *row,
@@ -254,7 +248,7 @@ row_activated_cb (GtkListBox         *box,
 
   dialog->update_view_tag = TRUE;
 
-  ephy_encoding_row = EPHY_ENCODING_ROW (gtk_bin_get_child (GTK_BIN (row)));
+  ephy_encoding_row = EPHY_ENCODING_ROW (gtk_list_box_row_get_child (row));
   ephy_encoding = ephy_encoding_row_get_encoding (ephy_encoding_row);
   selected_encoding = ephy_encoding_get_encoding (ephy_encoding);
 
@@ -322,7 +316,7 @@ static void
 add_list_item (EphyEncoding *encoding,
                GtkListBox   *list_box)
 {
-  gtk_container_add (GTK_CONTAINER (list_box), GTK_WIDGET (ephy_encoding_row_new (encoding)));
+  gtk_list_box_append (GTK_LIST_BOX (list_box), GTK_WIDGET (ephy_encoding_row_new (encoding)));
 }
 
 static int
@@ -392,7 +386,7 @@ ephy_encoding_dialog_constructed (GObject *object)
     recent = g_list_sort (recent, (GCompareFunc)sort_encodings);
     g_list_foreach (recent, (GFunc)add_list_item, dialog->recent_list_box);
   } else
-    gtk_widget_hide (dialog->recent_box);
+    gtk_widget_set_visible (dialog->recent_box, FALSE);
 
   /* related */
   if (dialog->selected_encoding != NULL) {
@@ -406,7 +400,7 @@ ephy_encoding_dialog_constructed (GObject *object)
     related = g_list_sort (related, (GCompareFunc)sort_encodings);
     g_list_foreach (related, (GFunc)add_list_item, dialog->related_list_box);
   } else
-    gtk_widget_hide (dialog->related_box);
+    gtk_widget_set_visible (dialog->related_box, FALSE);
 
   /* update list_boxes */
   sync_encoding_against_embed (dialog);
@@ -491,8 +485,7 @@ ephy_encoding_dialog_class_init (EphyEncodingDialogClass *klass)
 
   obj_properties[PROP_PARENT_WINDOW] =
     g_param_spec_object ("parent-window",
-                         "Parent window",
-                         "Parent window",
+                         NULL, NULL,
                          EPHY_TYPE_WINDOW,
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
@@ -510,16 +503,16 @@ ephy_encoding_dialog_class_init (EphyEncodingDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, EphyEncodingDialog, related_box);
 
   gtk_widget_class_bind_template_callback (widget_class, default_switch_toggled_cb);
-  gtk_widget_class_bind_template_callback (widget_class, ephy_encoding_dialog_response_cb);
   gtk_widget_class_bind_template_callback (widget_class, row_activated_cb);
   gtk_widget_class_bind_template_callback (widget_class, show_all_button_clicked_cb);
+
+  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_Escape, 0, "window.close", NULL);
 }
 
 EphyEncodingDialog *
 ephy_encoding_dialog_new (EphyWindow *parent)
 {
   return g_object_new (EPHY_TYPE_ENCODING_DIALOG,
-                       "use-header-bar", TRUE,
                        "parent-window", parent,
                        NULL);
 }
