@@ -32,7 +32,6 @@
 #include "ephy-search-engine-listbox.h"
 #include "ephy-shell.h"
 #include "ephy-web-app-utils.h"
-#include "webapp-additional-urls-dialog.h"
 
 #include "gnome-languages.h"
 #include <glib/gi18n.h>
@@ -42,6 +41,11 @@ enum {
   COL_LANG_CODE
 };
 
+enum {
+  WEBAPP_ADDITIONAL_URLS_ROW_ACTIVATED,
+  LAST_SIGNAL
+};
+
 struct _PrefsGeneralPage {
   AdwPreferencesPage parent_instance;
 
@@ -71,7 +75,11 @@ struct _PrefsGeneralPage {
   GtkWidget *download_folder_label;
 
   /* Search Engines */
+  GtkWidget *default_search_engines;
+  GtkWidget *standard_search_engine;
+  GtkWidget *incognito_search_engine;
   GtkWidget *search_engine_group;
+  EphySearchEngineListBox *search_engine_list_box;
 
   /* Session */
   GtkWidget *session_box;
@@ -92,6 +100,7 @@ struct _PrefsGeneralPage {
   /* Developer */
   AdwPreferencesGroup *dev_group;
   GtkWidget *show_developer_actions_row;
+  GtkWidget *always_show_full_url_row;
 
   GtkWindow *add_lang_dialog;
   GtkTreeView *add_lang_treeview;
@@ -99,6 +108,8 @@ struct _PrefsGeneralPage {
   GCancellable *cancellable;
 };
 
+static guint signals[LAST_SIGNAL];
+
 G_DEFINE_FINAL_TYPE (PrefsGeneralPage, prefs_general_page, ADW_TYPE_PREFERENCES_PAGE)
 
 static void
@@ -132,6 +143,24 @@ get_list_box_length (GtkListBox *list_box)
   return i;
 }
 
+static void
+language_editor_update_rows_movable (PrefsGeneralPage *general_page,
+                                     GtkListBox       *list_box)
+{
+  GtkListBoxRow *row;
+  int n_rows = get_list_box_length (list_box) - 1;
+  int i = 0;
+
+  while ((row = gtk_list_box_get_row_at_index (list_box, i++))) {
+    if (!EPHY_IS_LANG_ROW (row))
+      continue;
+
+    gtk_widget_action_set_enabled (GTK_WIDGET (row), "row.move-up", i > 0);
+    gtk_widget_action_set_enabled (GTK_WIDGET (row), "row.move-down", i < (n_rows - 1));
+    ephy_lang_row_set_movable (EPHY_LANG_ROW (row), n_rows > 1);
+  }
+}
+
 static void
 language_editor_update_pref (PrefsGeneralPage *general_page)
 {
@@ -238,6 +267,7 @@ language_editor_delete_button_clicked_cb (EphyLangRow      *row,
   gtk_list_box_remove (GTK_LIST_BOX (general_page->lang_listbox), GTK_WIDGET (row));
   language_editor_update_pref (general_page);
   language_editor_update_state (general_page);
+  language_editor_update_rows_movable (general_page, GTK_LIST_BOX (general_page->lang_listbox));
 }
 
 static void
@@ -290,6 +320,7 @@ language_editor_add (PrefsGeneralPage *general_page,
   g_signal_connect (row, "move-row", G_CALLBACK (language_editor_move_row_cb), general_page);
 
   gtk_list_box_insert (GTK_LIST_BOX (general_page->lang_listbox), row, len - 1);
+  language_editor_update_rows_movable (general_page, GTK_LIST_BOX (general_page->lang_listbox));
 }
 
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
@@ -709,6 +740,13 @@ prefs_general_page_on_pd_close_request (PrefsGeneralPage *general_page)
   }
 }
 
+/* Used by EphyAddOpenSearchButton to scroll to the just added engine. */
+EphySearchEngineListBox *
+prefs_general_page_get_search_engine_list_box (PrefsGeneralPage *general_page)
+{
+  return general_page->search_engine_list_box;
+}
+
 static void
 on_webapp_icon_row_activated (GtkWidget        *button,
                               PrefsGeneralPage *general_page)
@@ -885,16 +923,51 @@ custom_homepage_set_mapping (const GValue       *value,
 }
 
 static void
-on_manage_webapp_additional_urls_row_activated (GtkWidget        *button,
+on_manage_webapp_additional_urls_row_activated (GtkWidget        *row,
                                                 PrefsGeneralPage *general_page)
 {
-  EphyWebappAdditionalURLsDialog *urls_dialog;
-  GtkWindow *window;
+  g_signal_emit (general_page, signals[WEBAPP_ADDITIONAL_URLS_ROW_ACTIVATED], 0);
+}
 
-  urls_dialog = ephy_webapp_additional_urls_dialog_new ();
-  window = GTK_WINDOW (gtk_widget_get_root (GTK_WIDGET (general_page)));
+static void
+search_engine_row_expanded_cb (GObject  *object,
+                               gpointer  user_data)
+{
+  EphySearchEngineRow *row = EPHY_SEARCH_ENGINE_ROW (object);
 
-  adw_dialog_present (ADW_DIALOG (urls_dialog), GTK_WIDGET (window));
+  ephy_search_engine_row_focus_name_entry (row);
+  g_signal_handlers_disconnect_by_func (object, search_engine_row_expanded_cb, user_data);
+}
+
+#define EMPTY_NEW_SEARCH_ENGINE_NAME (_("New search engine"))
+static void
+on_add_search_engine_button_clicked_cb (GtkButton *button,
+                                        gpointer   user_data)
+{
+  PrefsGeneralPage *page = EPHY_PREFS_GENERAL_PAGE (user_data);
+  EphySearchEngineManager *manager = ephy_embed_shell_get_search_engine_manager (ephy_embed_shell_get_default ());
+  EphySearchEngine *old_empty_engine = ephy_search_engine_manager_find_engine_by_name (manager, EMPTY_NEW_SEARCH_ENGINE_NAME);
+
+  if (old_empty_engine) {
+    EphySearchEngineRow *row =
+      ephy_search_engine_list_box_find_row_for_engine (page->search_engine_list_box, old_empty_engine);
+
+    /* Change the focus to an arbitrary widget we have available here first,
+     * because the scroll-to-focus feature won't work if the focus is grabbed on
+     * an already focused widget.
+     */
+    ephy_search_engine_row_focus_bang_entry (row);
+    adw_expander_row_set_expanded (ADW_EXPANDER_ROW (row), FALSE);
+    adw_expander_row_set_expanded (ADW_EXPANDER_ROW (row), TRUE);
+    g_signal_connect (row, "fully-expanded", G_CALLBACK (search_engine_row_expanded_cb), NULL);
+  } else {
+    g_autoptr (EphySearchEngine) empty_engine =
+      g_object_new (EPHY_TYPE_SEARCH_ENGINE,
+                    "name", EMPTY_NEW_SEARCH_ENGINE_NAME,
+                    "url", "https://www.example.com/search?q=%s",
+                    NULL);
+    ephy_search_engine_manager_add_engine (manager, empty_engine);
+  }
 }
 
 static void
@@ -908,6 +981,13 @@ prefs_general_page_class_init (PrefsGeneralPageClass *klass)
   gtk_widget_class_set_template_from_resource (widget_class,
                                                "/org/gnome/epiphany/gtk/prefs-general-page.ui");
 
+  signals[WEBAPP_ADDITIONAL_URLS_ROW_ACTIVATED] =
+    g_signal_new ("webapp-additional-row-activated",
+                  EPHY_TYPE_PREFS_GENERAL_PAGE,
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL, NULL,
+                  G_TYPE_NONE, 0);
+
   /* Web Application */
   gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, webapp_box);
   gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, webapp_icon);
@@ -933,7 +1013,11 @@ prefs_general_page_class_init (PrefsGeneralPageClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, download_folder_label);
 
   /* Search Engines */
+  gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, default_search_engines);
+  gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, standard_search_engine);
+  gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, incognito_search_engine);
   gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, search_engine_group);
+  gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, search_engine_list_box);
 
   /* Session */
   gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, session_box);
@@ -954,12 +1038,14 @@ prefs_general_page_class_init (PrefsGeneralPageClass *klass)
   /* Developer */
   gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, dev_group);
   gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, show_developer_actions_row);
+  gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, always_show_full_url_row);
 
   /* Signals */
   gtk_widget_class_bind_template_callback (widget_class, on_webapp_icon_row_activated);
   gtk_widget_class_bind_template_callback (widget_class, on_webapp_entry_changed);
-  gtk_widget_class_bind_template_callback (widget_class, on_manage_webapp_additional_urls_row_activated);
   gtk_widget_class_bind_template_callback (widget_class, download_folder_row_activated_cb);
+  gtk_widget_class_bind_template_callback (widget_class, on_manage_webapp_additional_urls_row_activated);
+  gtk_widget_class_bind_template_callback (widget_class, on_add_search_engine_button_clicked_cb);
 }
 
 static void
@@ -988,6 +1074,125 @@ init_lang_listbox (PrefsGeneralPage *general_page)
       }
     }
   }
+
+  language_editor_update_state (general_page);
+}
+
+static gboolean
+default_search_engine_get_mapping (GValue   *value,
+                                   GVariant *variant,
+                                   gpointer  user_data)
+{
+  EphySearchEngineManager *manager = ephy_embed_shell_get_search_engine_manager (ephy_embed_shell_get_default ());
+  const char *text = g_variant_get_string (variant, NULL);
+
+  g_assert (text);
+
+  for (guint i = 0; i < g_list_model_get_n_items (G_LIST_MODEL (manager)); i++) {
+    EphySearchEngine *engine = g_list_model_get_item (G_LIST_MODEL (manager), i);
+
+    if (g_strcmp0 (ephy_search_engine_get_name (engine), text) == 0) {
+      g_value_set_uint (value, i);
+      return TRUE;
+    }
+  }
+
+  return FALSE;
+}
+
+static GVariant *
+default_search_engine_set_mapping (const GValue       *value,
+                                   const GVariantType *expected_type,
+                                   gpointer            user_data)
+{
+  EphySearchEngineManager *manager = ephy_embed_shell_get_search_engine_manager (ephy_embed_shell_get_default ());
+  EphySearchEngine *engine;
+  guint i = g_value_get_uint (value);
+
+  if (i >= g_list_model_get_n_items (G_LIST_MODEL (manager)))
+    return NULL;
+
+  engine = g_list_model_get_item (G_LIST_MODEL (manager), i);
+  g_assert (engine);
+
+  ephy_search_engine_manager_set_default_engine (manager, engine);
+  return g_variant_new_string (ephy_search_engine_get_name (engine));
+}
+
+static gboolean
+incognito_search_engine_get_mapping (GValue   *value,
+                                     GVariant *variant,
+                                     gpointer  user_data)
+{
+  EphySearchEngineManager *manager = ephy_embed_shell_get_search_engine_manager (ephy_embed_shell_get_default ());
+  const char *text = g_variant_get_string (variant, NULL);
+
+  g_assert (text);
+
+  for (guint i = 0; i < g_list_model_get_n_items (G_LIST_MODEL (manager)); i++) {
+    EphySearchEngine *engine = g_list_model_get_item (G_LIST_MODEL (manager), i);
+
+    if (g_strcmp0 (ephy_search_engine_get_name (engine), text) == 0) {
+      g_value_set_uint (value, i);
+      return TRUE;
+    }
+  }
+
+  return FALSE;
+}
+
+static GVariant *
+incognito_search_engine_set_mapping (const GValue       *value,
+                                     const GVariantType *expected_type,
+                                     gpointer            user_data)
+{
+  EphySearchEngineManager *manager = ephy_embed_shell_get_search_engine_manager (ephy_embed_shell_get_default ());
+  EphySearchEngine *engine;
+  guint i = g_value_get_uint (value);
+
+  if (i >= g_list_model_get_n_items (G_LIST_MODEL (manager)))
+    return NULL;
+
+  engine = g_list_model_get_item (G_LIST_MODEL (manager), i);
+  g_assert (engine);
+
+  ephy_search_engine_manager_set_incognito_engine (manager, engine);
+  return g_variant_new_string (ephy_search_engine_get_name (engine));
+}
+
+static void
+init_search_engines (PrefsGeneralPage *general_page)
+{
+  EphySearchEngineManager *manager = ephy_embed_shell_get_search_engine_manager (ephy_embed_shell_get_default ());
+  GtkExpression *expression;
+
+  /* Default */
+  expression = gtk_property_expression_new (EPHY_TYPE_SEARCH_ENGINE, NULL, "name");
+  adw_combo_row_set_expression (ADW_COMBO_ROW (general_page->standard_search_engine), expression);
+  adw_combo_row_set_model (ADW_COMBO_ROW (general_page->standard_search_engine), G_LIST_MODEL (manager));
+
+  g_settings_bind_with_mapping (EPHY_SETTINGS_MAIN,
+                                EPHY_PREFS_DEFAULT_SEARCH_ENGINE,
+                                general_page->standard_search_engine,
+                                "selected",
+                                G_SETTINGS_BIND_DEFAULT,
+                                default_search_engine_get_mapping,
+                                default_search_engine_set_mapping,
+                                NULL, NULL);
+
+  /* Incognito */
+  expression = gtk_property_expression_new (EPHY_TYPE_SEARCH_ENGINE, NULL, "name");
+  adw_combo_row_set_expression (ADW_COMBO_ROW (general_page->incognito_search_engine), expression);
+  adw_combo_row_set_model (ADW_COMBO_ROW (general_page->incognito_search_engine), G_LIST_MODEL (manager));
+
+  g_settings_bind_with_mapping (EPHY_SETTINGS_MAIN,
+                                EPHY_PREFS_INCOGNITO_SEARCH_ENGINE,
+                                general_page->incognito_search_engine,
+                                "selected",
+                                G_SETTINGS_BIND_DEFAULT,
+                                incognito_search_engine_get_mapping,
+                                incognito_search_engine_set_mapping,
+                                NULL, NULL);
 }
 
 static void
@@ -1155,7 +1360,14 @@ setup_general_page (PrefsGeneralPage *general_page)
                    "active",
                    G_SETTINGS_BIND_DEFAULT);
 
+  g_settings_bind (web_settings,
+                   EPHY_PREFS_WEB_ALWAYS_SHOW_FULL_URL,
+                   general_page->always_show_full_url_row,
+                   "active",
+                   G_SETTINGS_BIND_DEFAULT);
+
   init_lang_listbox (general_page);
+  init_search_engines (general_page);
 }
 
 static void
@@ -1178,6 +1390,8 @@ prefs_general_page_init (PrefsGeneralPage *general_page)
   gtk_widget_set_visible (general_page->webapp_icon_row, !ephy_is_running_inside_sandbox ());
   gtk_widget_set_visible (general_page->webapp_url_row, !ephy_is_running_inside_sandbox ());
   gtk_widget_set_visible (general_page->webapp_title_row, !ephy_is_running_inside_sandbox ());
+  gtk_widget_set_visible (general_page->default_search_engines,
+                          mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
   gtk_widget_set_visible (general_page->homepage_box,
                           mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
   gtk_widget_set_visible (general_page->search_engine_group,
